/*
	Copyright (c) 2004 PXI Project Team

	Permission is hereby granted, free of charge, to any person obtaining a copy of
	this software and associated documentation files (the "Software"), to deal in the
	Software without restriction, including without limitation the rights to use,
	copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
	Software, and to permit persons to whom the Software is furnished to do so,
	subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies
	or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
	INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
	PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
	OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <string>
#include <vector>
#include "memoryblock.h"
#include "stringmaniplib.h"
#include "PXITypes.h"
#include "PXIVariant.h"
using namespace std;

PXIVariant::PXIVariant(void)
{
	clear();
}

PXIVariant::PXIVariant(const PXIVariant & pv)
{
	varType = pv.varType;
	booleanValue = pv.booleanValue;
	integerValue = pv.integerValue;
	colorValue = pv.colorValue;
	rectangleValue = pv.rectangleValue;
	pointValue = pv.pointValue;
	point3dValue = pv.point3dValue;
	dffValue = pv.dffValue;
	stringValue = pv.stringValue;
	binaryValue = pv.binaryValue;
	integerListValue = pv.integerListValue;
}

const PXIVariant & PXIVariant::operator= (const PXIVariant & pv)
{
	varType = pv.varType;
	booleanValue = pv.booleanValue;
	integerValue = pv.integerValue;
	colorValue = pv.colorValue;
	rectangleValue = pv.rectangleValue;
	pointValue = pv.pointValue;
	point3dValue = pv.point3dValue;
	dffValue = pv.dffValue;
	stringValue = pv.stringValue;
	binaryValue = pv.binaryValue;
	integerListValue = pv.integerListValue;
	return *this;
}

void PXIVariant::clear(void)
{
	color c;
	rect r;
	point p;
	point3d p3;
	dffref d;
	memoryblock m;
	vector<int> v;
	varType = PXINIL;
	booleanValue = false;
	integerValue = 0;
	colorValue = c;
	rectangleValue = r;
	pointValue = p;
	point3dValue = p3;
	dffValue = d;
	stringValue = "";
	binaryValue = m;
	integerListValue = v;
}

void PXIVariant::setBoolValue(bool b)
{
	clear();
	varType = PXIBOOL;
	booleanValue = b;
	if (b) {
		integerValue = 1;
		stringValue = "true";
		color c(0,0,0);
		colorValue = c;
	} else {
		integerValue = 0;
		stringValue = "false";
		color c(255,255,255);
		colorValue = c;
	}
	memoryblock m(1);
	m.setByte(0,integerValue);
	binaryValue = m;
	vector<int> v;
	v.push_back(integerValue);
	integerListValue = v;
}

void PXIVariant::setIntValue(int i)
{
	clear();
	varType = PXIINT;
	integerValue = i;
	booleanValue = (i != 0);
	stringValue = str(i);
	color c(i,i,i);
	colorValue = c;
	memoryblock m(4);
	m.setLong(0,i);
	binaryValue = m;
	vector<int> v;
	v.push_back(i);
	integerListValue = v;
}

void PXIVariant::setColorValue(color c)
{
	clear();
	varType = PXICOLOR;
	colorValue = c;
	booleanValue = (((c.red + c.green + c.blue) / 3) < 32768);
	integerValue = ((c.red + c.green + c.blue) / 3);
	point3d p(c.red, c.green, c.blue);
	point3dValue = p;
	stringValue = str(c.red) + (string)"," + str(c.green) + (string)"," + str(c.blue);
	memoryblock m(6);
	m.setShort(0,c.red);
	m.setShort(2,c.green);
	m.setShort(4,c.blue);
	binaryValue = m;
	vector<int> v;
	v.push_back(c.red);
	v.push_back(c.green);
	v.push_back(c.blue);
	integerListValue = v;
}

void PXIVariant::setColorValue(int r, int g, int b)
{
	color c(r,g,b);
	setColorValue(c);
}

void PXIVariant::setRectValue(rect r)
{
	clear();
	varType = PXIRECT;
	rectangleValue = r;
	point p( (r.left + r.right)/2 , (r.top + r.bottom)/2 );
	pointValue = p;
	point3d p3( (r.left + r.right)/2 , (r.top + r.bottom)/2 , 0 );
	point3dValue = p3;
	stringValue = str(r.left) + (string)"," + str(r.top) + (string)"," + str(r.right) + (string)"," + str(r.bottom);
	memoryblock m(8);
	m.setShort(0,r.top);
	m.setShort(2,r.left);
	m.setShort(4,r.bottom);
	m.setShort(6,r.right);
	binaryValue = m;
	vector<int> v;
	v.push_back(r.left);
	v.push_back(r.top);
	v.push_back(r.right);
	v.push_back(r.bottom);
	integerListValue = v;
}

void PXIVariant::setRectValue(int l, int t, int r, int b)
{
	rect rr(l,t,r,b);
	setRectValue(rr);
}

void PXIVariant::setPointValue(point p)
{
	clear();
	varType = PXIPOINT;
	pointValue = p;
	booleanValue = ( (p.x != 0) || (p.y != 0) );
	rect r(p.x, p.y, p.x, p.y);
	rectangleValue = r;
	point3d p3(p.x, p.y, 0);
	point3dValue = p3;
	stringValue = str(p.x) + (string)"," + str(p.y);
	memoryblock m(4);
	m.setShort(0,p.y);
	m.setShort(2,p.x);
	binaryValue = m;
	vector<int> v;
	v.push_back(p.x);
	v.push_back(p.y);
	integerListValue = v;
}

void PXIVariant::setPointValue(int x, int y)
{
	point p(x,y);
	setPointValue(p);
}

void PXIVariant::setPoint3DValue(point3d p)
{
	clear();
	varType = PXI3DPOINT;
	point3dValue = p;
	booleanValue = ( (p.x != 0) || (p.y != 0) || (p.z != 0) );
	point p2(p.x, p.y);
	pointValue = p2;
	stringValue = str(p.x) + (string)"," + str(p.y) + (string)"," + str(p.z);
	memoryblock m(6);
	m.setShort(0,p.z);
	m.setShort(2,p.y);
	m.setShort(4,p.x);
	binaryValue = m;
	vector<int> v;
	v.push_back(p.x);
	v.push_back(p.y);
	v.push_back(p.z);
	integerListValue = v;
}

void PXIVariant::setPoint3DValue(int x, int y, int z)
{
	point3d p(x,y,z);
	setPoint3DValue(p);
}

void PXIVariant::setDffRefValue(dffref d)
{
	clear();
	varType = PXIDFFREF;
	dffValue = d;
	integerValue = d.id;
	stringValue = trim(d.symbol) + (string)" " + str(d.id);
	memoryblock m(12);
	string ss = left(trim(d.symbol) + (string)"        ", 8);
	m.setStringValue(0,8,ss.c_str());
	m.setLong(8,d.id);
	binaryValue = m;
}

void PXIVariant::setDffRefValue(string s, int i)
{
	dffref d(s,i);
	setDffRefValue(d);
}

void PXIVariant::setStringValue(string s)
{
	clear();
	varType = PXISTRING;
	stringValue = s;
	integerValue = val(s);
	booleanValue = cie(s, (string)"true");
	string s2 = trim(s);
	int i;
	int m = countfields(s2, (string)",");
	color c;
	rect r;
	point p;
	point3d p3;
	dffref d;
	vector<int> v;
	switch(m){
	case 1:
		c.red = integerValue;
		c.green = integerValue;
		c.blue = integerValue;
		colorValue = c;
	case 2:
		p.x = val(nthfield(s2, (string)",", 1));
		p.y = val(nthfield(s2, (string)",", 2));
		pointValue = p;
		p3.x = p.x;
		p3.y = p.y;
		p3.z = 0;
		point3dValue = p3;
		r.left = p.x;
		r.top = p.y;
		r.right = p.x;
		r.bottom = p.y;
		rectangleValue = r;
		break;
	case 3:
		p3.x = val(nthfield(s2, (string)",", 1));
		p3.y = val(nthfield(s2, (string)",", 2));
		p3.z = val(nthfield(s2, (string)",", 3));
		point3dValue = p3;
		c.red = p3.x;
		c.green = p3.y;
		c.blue = p3.z;
		colorValue = c;
		break;
	case 4:
		r.left = val(nthfield(s2, (string)",", 1));
		r.top = val(nthfield(s2, (string)",", 2));
		r.right = val(nthfield(s2, (string)",", 3));
		r.bottom = val(nthfield(s2, (string)",", 4));
		rectangleValue = r;
		p.x = (r.left + r.right) / 2;
		p.y = (r.top + r.bottom) / 2;
		pointValue = p;
		break;
	}
	for (i=1; i<=m; i++)
	{
		v.push_back(val(nthfield(s2, (string)",", i)));
	}
	integerListValue = v;
	m = countfields(s2, (string)" ");
	string s3 = nthfield(s2, (string)" ", m);
	string s4 = trim(left(s2, (s2.size() - s3.size()) ))
	if ( (m > 1) && (s4.size() < 9) )
	{
		d.symbol = left(s4 + (string)"        " , 8);
		d.id = val(s3);
		dffValue = d;
	}
	memoryblock mb(s.size()+1);
	mb.setStringValue(0, s.size(), s.c_str());
	mb.setByte(s.size(), 0);
	binaryValue = mb;
}

void PXIVariant::setBinaryValue(memoryblock m)
{
	clear();
	varType = PXIBINARY;
	binaryValue = m;
	color c;
	rect r;
	dffref d;
	point p;
	point3d p3;
	vector<int> v;
	switch(m.getSize()){
	case 1:		/* boolean, integer */
		integerValue = m.getByte(0);
		booleanValue = (integerValue != 0);
		c.red = integerValue*257;
		c.green = integerValue*257;
		c.blue = integerValue*257;
		colorValue = c;
		break;
	case 2:		/* integer */
		integerValue = m.getShort(0);
		booleanValue = (integerValue != 0);
		c.red = integerValue;
		c.green = integerValue;
		c.blue = integerValue;
		colorValue = c;
		break;
	case 4:		/* integer, point */
		integerValue = m.getLong(0);
		booleanValue = (integerValue != 0);
		p.y = m.getShort(0);
		p.x = m.getShort(2);
		pointValue = p;
		p3.x = p.x;
		p3.y = p.y;
		p3.z = 0;
		point3dValue = p3;
		r.left = p.x;
		r.top = p.y;
		r.right = p.x;
		r.bottom = p.y;
		rectangleValue = r;
		break;
	case 6:		/* color, point3d */
		p3.z = m.getShort(0);
		p3.y = m.getShort(2);
		p3.x = m.getShort(4);
		point3dValue = p3;
		c.red = m.getUShort(0);
		c.green = m.getUShort(2);
		c.blue = m.getUShort(4);
		colorValue = c;
		booleanValue = ( (p3.x != 0) || (p3.y != 0) || (p3.z != 0) );
		integerValue = ((c.red + c.green + c.blue) / 3);
		break;
	case 8:		/* rect */
		r.top = m.getShort(0);
		r.left = m.getShort(2);
		r.bottom = m.getShort(4);
		r.right = m.getShort(6);
		rectangleValue = r;
		p.x = (r.left + r.right) / 2;
		p.y = (r.top + r.bottom) / 2;
		pointValue = p;
		p3.x = p.x;
		p3.y = p.y;
		p3.z = 0;
		point3dValue = p3;
		booleanValue = ( (r.left != 0) || (r.top != 0) || (r.right != 0) || (r.bottom != 0) );
		break;
	case 10:	/* dffref */
		integerValue = m.getShort(8);
		d.id = integerValue;
		d.symbol = m.getStringValueAsString(0, 8);
		dffValue = d;
		break;
	case 12:	/* dffref */
		integerValue = m.getLong(8);
		d.id = integerValue;
		d.symbol = m.getStringValueAsString(0, 8);
		dffValue = d;
		break;
	}
	int mm = m.getSize();
	int i;
	for (i=0; i<mm; i+=4)
	{
		v.push_back(m.getLong(i));
	}
	integerListValue = v;
	stringValue = m.getStringValueAsString(0, mm);
}

void PXIVariant::setIntListValue(vector<int> v)
{
	clear();
	varType = PXIINTLIST;
	integerListValue = v;
	string s;
	memoryblock m;
	int i;
	vector<int>::iterator ii;
	m.setSize( ((int)v.size()) * 4);
	for (ii = v.begin(), i = 0; ii < v.end(); ii++, i+=4)
	{
		s += (string)",";
		s += str(*ii);
		m.setLong(i, *ii);
	}
	stringValue = mid(s,2);
	binaryValue = m;
	switch((int)v.size()){
	case 1:
		i = v[0];
		integerValue = i;
		booleanValue = (i != 0);
		color c(i,i,i);
		colorValue = c;
		break;
	case 2:
		point p(v[0], v[1]);
		pointValue = p;
		booleanValue = ( (p.x != 0) || (p.y != 0) );
		rect r(p.x, p.y, p.x, p.y);
		rectangleValue = r;
		point3d p3(p.x, p.y, 0);
		point3dValue = p3;
		break;
	case 3:
		color c(v[0], v[1], v[2]);
		colorValue = c;
		booleanValue = (((c.red + c.green + c.blue) / 3) < 32768);
		integerValue = ((c.red + c.green + c.blue) / 3);
		point3d p(c.red, c.green, c.blue);
		point3dValue = p;
		break;
	case 4:
		rect r(v[0], v[1], v[2], v[3]);
		rectangleValue = r;
		point p( (r.left + r.right)/2 , (r.top + r.bottom)/2 );
		pointValue = p;
		point3d p3( (r.left + r.right)/2 , (r.top + r.bottom)/2 , 0 );
		point3dValue = p3;
		break;
	}
}

bool getBoolValue(void)
{
	return booleanValue;
}

int getIntValue(void)
{
	return integerValue;
}

color getColorValue(void)
{
	return colorValue;
}

rect getRectValue(void)
{
	return rectangleValue;
}

point getPointValue(void)
{
	return pointValue;
}

point3d getPoint3DValue(void)
{
	return point3dValue;
}

dffref getDffRefValue(void)
{
	return dffValue;
}

string getStringValue(void)
{
	return stringValue;
}

memoryblock getBinaryValue(void)
{
	return binaryValue;
}

vector<int> getIntListValue(void)
{
	return integerListValue;
}

PXIVariantType getVarType(void)
{
	return varType;
}

bool isBool(void)
{
	return ( (varType == PXIBOOL) || (varType == PXIINT) );
}

bool isInt(void)
{
	return ( (varType == PXIBOOL) || (varType == PXIINT) || ((varType == PXIINTLIST) && (integerListValue.size() == 1)) );
}

bool isColor(void)
{
	return ( (varType == PXICOLOR) || (varType == PXI3DPOINT) || ((varType == PXIINTLIST) && (integerListValue.size() == 3)) );
}

bool isRect(void)
{
	return ( (varType == PXIRECT) || ((varType == PXIINTLIST) && (integerListValue.size() == 4)) );
}

bool isPoint(void)
{
	return ( (varType == PXIPOINT) || (varType == PXIRECT) || ((varType == PXIINTLIST) && (integerListValue.size() == 2)) );
}

bool isPoint3D(void)
{
	return ( (varType == PXICOLOR) || (varType == PXI3DPOINT) || (varType == PXIPOINT) || ((varType == PXIINTLIST) && (integerListValue.size() == 3)) );
}

bool isDffRef(void)
{
	return ( (varType == PXIDFFREF) );
}

bool isString(void)
{
	return true;
}

bool isBinary(void)
{
	return ( binaryValue.getSize() != 0 );
}

bool isIntList(void)
{
	return (! integerListValue.empty() );
}

void makeBool(void)
{
	setBoolValue(booleanValue);
}

void makeInt(void)
{
	setIntValue(integerValue);
}

void makeColor(void)
{
	setColorValue(colorValue);
}

void makeRect(void)
{
	setRectValue(rectangleValue);
}

void makePoint(void)
{
	setPointValue(pointValue);
}

void makePoint3D(void)
{
	setPoint3DValue(point3dValue);
}

void makeDffRef(void)
{
	setDffRefValue(dffValue);
}

void makeString(void)
{
	setStringValue(stringValue);
}

void makeBinary(void)
{
	setBinaryValue(binaryValue);
}

void makeIntList(void)
{
	setIntListValue(integerListValue);
}
