/*
	Copyright (c) 2004 PXI Project Team

	Permission is hereby granted, free of charge, to any person obtaining a copy of
	this software and associated documentation files (the "Software"), to deal in the
	Software without restriction, including without limitation the rights to use,
	copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
	Software, and to permit persons to whom the Software is furnished to do so,
	subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies
	or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
	INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
	PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
	OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef _____PXIVARIANT_____
#define _____PXIVARIANT_____

#include <string>
#include <vector>
#include "memoryblock.h"
#include "PXITypes.h"
using namespace std;

class PXIVariant
{
	public:

	PXIVariant(void);
	PXIVariant(const PXIVariant &);
	const PXIVariant & operator= (const PXIVariant &);

	void clear(void);

	void setBoolValue(bool);
	void setIntValue(int);
	void setColorValue(color);
	void setColorValue(int, int, int);
	void setRectValue(rect);
	void setRectValue(int, int, int, int);
	void setPointValue(point);
	void setPointValue(int, int);
	void setPoint3DValue(point3d);
	void setPoint3DValue(int, int, int);
	void setDffRefValue(dffref);
	void setDffRefValue(string, int);
	void setStringValue(string);
	void setBinaryValue(memoryblock);
	void setIntListValue(vector<int>);

	bool	getBoolValue(void);
	int	getIntValue(void);
	color	getColorValue(void);
	rect	getRectValue(void);
	point	getPointValue(void);
	point3d	getPoint3DValue(void);
	dffref	getDffRefValue(void);
	string	getStringValue(void);
	memoryblock	getBinaryValue(void);
	vector<int>	getIntListValue(void);

	bool isBool(void);
	bool isInt(void);
	bool isColor(void);
	bool isRect(void);
	bool isPoint(void);
	bool isPoint3D(void);
	bool isDffRef(void);
	bool isString(void);
	bool isBinary(void);
	bool isIntList(void);

	void makeBool(void);
	void makeInt(void);
	void makeColor(void);
	void makeRect(void);
	void makePoint(void);
	void makePoint3D(void);
	void makeDffRef(void);
	void makeString(void);
	void makeBinary(void);
	void makeIntList(void);

	enum PXIVariantType { PXINIL, PXIBOOL, PXIINT, PXICOLOR, PXIRECT, PXIPOINT, PXI3DPOINT, PXIDFFREF, PXISTRING, PXIBINARY, PXIINTLIST } ;

	PXIVariantType getVarType(void);

	private:

	PXIVariantType	varType;

	bool	booleanValue;
	int	integerValue;
	color	colorValue;
	rect	rectangleValue;
	point	pointValue;
	point3d	point3dValue;
	dffref	dffValue;
	string	stringValue;
	memoryblock	binaryValue;
	vector<int>	integerListValue;
};

#endif